<?php
/*
    LWAT, LDAP Webbased Administration Tool
    Copyright (C) 2007 Finn-Arne Johansen <faj@bzz.no> BzzWare AS, Norway

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

function loadLocale () {
    $found_locale = $_COOKIE['locale'] ;
    
    if ($found_locale == "" && $handle = opendir ('./locales')) {
	$all_locales = array () ; 
	while (false !== ($file = readdir ($handle))) {
	    if (is_dir ('./locales/' . $file . '/LC_MESSAGES')) {
		$all_locales = array_merge ($all_locales, array ($file)) ; 
	    }
	}
	closedir ($handle) ; 
	$lang_accept = array () ; 
	$lang_accept = explode (",", $_SERVER['HTTP_ACCEPT_LANGUAGE']); 
	for ($i = 0 ; $i < count ($lang_accept) ; $i++ ) {
	    $lang_accept[$i] = split(";", $lang_accept[$i]) ; 
	    $lang_accept[$i] = $lang_accept[$i][0] ; 
	}
	if (!empty($lang_accept[0]))
	foreach ($lang_accept as $lang) {
	    if (!isset ($locale_exact)) {
		foreach ($all_locales as $locales) {
		    if ($locales == $lang)
			$locale_exact = $lang ; 
		    elseif (!isset ($locale_match)) {
			if (strpos ($locales, $lang) === 0)
			    $locale_match = $locales ; 
		    }
		}
	    }
	}
	if (isset ($locale_exact)) 
	    $found_locale=$locale_exact ; 
	elseif (isset ($locale_match))
	    $found_locale=$locale_match ; 
	if (isset ($found_locale))
	    setcookie ( 'locale', $found_locale) ;
	unset ($all_locales, $lang_accept, $locale_match, $locale_exact, $lang, $locales) ; 
    }
	
    if (isset ($found_locale)) {
	setlocale (LC_ALL, $found_locale); 
	bindtextdomain ("lwat", "./locales/"); 
	bind_textdomain_codeset ("lwat", "UTF-8");
	textdomain ("lwat"); 
    }
}

function xorstring ($key, $xor) {
    $keylen=strlen ($key) ; 
    if (! $keylen) return "" ; 
    $newstr="" ; 
    $index=0 ; 
    $keyindex = $index ;
    while ($index < strlen ($xor)) {
        $newstr .= chr(ord($xor[$index]) ^ ord ($key[$keyindex])) ; 
        $index ++ ;
        $keyindex= $index % $keylen ;
    }
    if ($keyindex == 0) {
        $last=strpos ($newstr, chr(0)); 
        if ($last === false) { 
            return $newstr ;
        }
        return(substr ($newstr, 0, $last)) ; 
    }
    while ($keyindex != 0) {
        $newstr .= $key[$keyindex] ; 
        $index ++ ;
        $keyindex= $index % $keylen ;
    }
    return ($newstr) ;
}

function readKey ($key) {
    //return htmlentities(trim($_POST[$key])) ; 
    return trim($_POST[$key]) ; 
}

function debug ($mixed) {
    echo "<pre>\n" ; 
    print_r ($mixed);
    echo "\n</pre>\n" ; 
}

function pwgen(){
    $pw = '';
    $c  = 'bcdfghjkmnprstvwzBCDFGHJKLMNPQRSTVW'; //consonants except hard to speak ones
    $v  = 'aeiouAEU';              //vowels
    $a  = $c.$v;                //both

    //use two syllables...
    for($i=0;$i < 2; $i++){
        $pw .= $c[rand(0, strlen($c)-1)];
        $pw .= $v[rand(0, strlen($v)-1)];
        $pw .= $a[rand(0, strlen($a)-1)];
    }
    //... and add a nice number
    $pw .= rand(10,99);

    return $pw;
}

function cryptgen($clear){
//prepare a salt
    $salt = md5(uniqid(rand(), true));

    $salt=substr($salt,0,4);
    return '{SSHA}'.base64_encode(pack("H*", sha1($clear.$salt)).$salt);
}

function ntlmgen ($clear) {
      $command = "echo -n " . $clear . " | perl -MCrypt::SmbHash -e 'print ntlmgen <>'" ;
      return exec ($command) ;
}

function getnextid ($connect, $base) {
    $looped = false ; 
    $want= array ("gidNumber", "uidNumber");
    $filter = "(&(gidNumber=*)(cn=nextID))" ;
    $result= @ldap_search ($connect, "ou=Variables," . $base, $filter, $want);
    $nextID=0 ;
    if ($result) {
        $entries = ldap_get_entries ($connect, $result) ;
        if ($entries["count"]) 
            $nextID=$entries[0]["gidnumber"][0]  ; 
    }
    do {
        if ($nextID < 10000) $nextID=10000 ;
        else if ($nextID > 59999) {
            if ($looped) {
                die ('Sorry, all user id\'s between 10000 and 59999 seems to have been taken') ;
            } else {
                $looped = true ;
                $nextID=10000 ; 
            }
        }
        else $nextID++ ;
	$filter = "(|(gidNumber=$nextID)(uidNumber=$nextID))" ;
	$result= ldap_search ($connect, $base, $filter, $want);
	$entries = ldap_get_entries ($connect, $result) ;
    } while  ($entries["count"]) ; 
    $info=array () ;
    $info["objectClass"] = array ("top", "posixGroup") ;
    $info["gidnumber"] = $nextID + 1 ;
    $info["cn"] = "newID" ;
    if (ldap_add ($connect,"cn=newID,ou=variables," . $base,$info)) {
        ldap_delete ($connect,"cn=nextID,ou=variables," . $base) ;
        ldap_rename ($connect,"cn=newID,ou=variables," . $base, "cn=nextID", "ou=variables," . $base, true) ;
        return $nextID ;
    } 
    die ("ERROR: Unable update nextID") ;
}

function get_sn ($newname) {
    $sn="" ;
    $allnames = explode (" ", $newname) ;
    foreach ($allnames as $name) {
        $sn=$name ;
    }
    return $sn;
}

function get_username ($connect, $newname)  {
    global $base ;

    $charset = mb_detect_encoding ($newname, 'UTF-8, ISO-8859-1') ; 
    switch ($charset) {
        case 'ISO-8859-1':
            $basename = $newname ; 
            break ; 
        default:
            $basename = mb_convert_encoding ($newname, 'ISO-8859-1', $charset); 
            break ;
    }
    for ($i=0 ; $i < strlen($basename); $i++) {
        switch (ord($basename[$i])) {
            case 192: case 193: case 194: case 195: case 196: case 197:
                $basename[$i] = "A" ; 
                break ;
            case 199:
                $basename[$i] = "C" ; 
                break ;
            case 198: case 200: case 201: case 202: case 203:           
                $basename[$i] = "E" ; 
                break ;
            case 204: case 205: case 206: case 207:              
                $basename[$i] = "I" ;
                break ;
            case 209 :                          
                $basename[$i] = "N" ;
                break ;
            case 208: case 210: case 211: case 212: case 213: case 214: case 216 :  
                $basename[$i] = "O" ; 
                break ;
            case 217: case 218: case 219: case 220 :              
                $basename[$i] = "U" ; 
                break ;
            case 221 :                          
                $basename[$i] = "Y" ; 
                break ;
            case 222 :                          
		$basename[$i] = "P" ; 
		break ;
            case 223 :                          
		$basename[$i] = "I" ; 
		break ;
            case 224: case 225: case 226: case 227: case 228: case 229 :
		$basename[$i] = "a" ; 
		break ;
            case 231 : 
		$basename[$i] = "c" ; 
		break ;
            case 230: case 232: case 233: case 234: case 235 :
		$basename[$i] = "e" ; 
		break ;
            case 236: case 237: case 238: case 239 :
		$basename[$i] = "i" ; 
		break ;
            case 241 : 
		$basename[$i] = "n" ; 
		break ;
            case 240: case 242: case 243: case 244: case 245: case 246: case 248 :  
		$basename[$i] = "o" ; 
		break ;
            case 249: case 250: case 251: case 252 : 
		$basename[$i] = "u" ; 
		break ;
            case 253: case 255 : 
		$basename[$i] = "y" ; 
		break ;
            case 254 :   
		$basename[$i] = "p" ; 
		break ;
	}
    }
    $basename = strtolower ($basename) ;
    $allnames = preg_split ("/[\s,.-]+/", $basename) ;
    $firstname = $allnames[0] ;
    foreach ($allnames as $name) $lastname=$name ;
    $username = substr ($firstname, 0,3) . substr ($lastname, 0,3);
    $filter = "(|(uid=$username)(cn=$username))" ;
    $result= ldap_search ($connect, $base, $filter);
    $entries = ldap_get_entries ($connect, $result) ;
    if (!$entries["count"]) 
	return $username ;
    $loop="aeiouy" ;
    while (ereg ("[aeiouy]", $loop)) {
        $restname="" ;
        for ($i=1 ; $i < (count($allnames)); $i++)
	    $restname .= $allnames[$i] . " " ; 
	while (ereg ("[aeiouy]", $loop)) {
	    for ($count=1 ; $count <= 6 ; $count++ ) {
		$addname="" ;
		foreach (explode (" ", $restname) as $name)
		    $addname .= substr ($name, 0, $count);
		while (strlen ($addname) > 6)
		    $addname=substr($addname, $count);
		$username= substr($firstname,0,8 - strlen($addname)) . $addname ; 
		$filter = "(|(uid=$username)(cn=$username))" ;
		$result= ldap_search ($connect, $base, $filter, array ());
		$entries = ldap_get_entries ($connect, $result) ;
		if (!$entries["count"]) 
		    return $username ;
	    }
	    $length=strlen($restname);
	    $loop="" ;
	    while ($length--) {
		switch ($restname[$length]) {
		    case "a": case "e": case "i": case "o": case "u": case "y":
			$loop=$restname[$length] ;
			$restname=substr($restname, 0,$length) . substr($restname, $length + 1) ;
			$length = 0 ;
			break ;
		}
	    }
	    $length=strlen($firstname);
	    $loop="" ;
	    while ($length--) {
		switch ($firstname[$length]) {
		    case "a": case "e": case "i": case "o": case "u": case "y":
			$loop=$firstname[$length] ;
			$firstname=substr($firstname, 0,$length) . substr($firstname, $length + 1) ;
			$length = 0 ;
			break ;
		}
	    }
	}
    }
    echo "Sorry, I'm unable to generate a unique username for $newname<BR>";
    return "" ;
}

function get_givenName ($newname) {
    $last="" ; 
    foreach (explode (" ", $newname) as $value) {
        $givenName .= $last ;
        $last = "$value" . " " ;
    }
    if (! $givenName) $givenName=$last ;
    return $givenName ;
}

function get_sambasid ($connect, $userid) {
    global $base ;
    $want= array ("sambasid");
    $filter = "(objectClass=sambaDomain)" ;
    $result= ldap_search ($connect, $base, $filter, $want);
    if (! $result) {
        echo "Failed to search for sambaSID<BR>" ;
        return "" ;
    }
    $entries = ldap_get_entries ($connect, $result) ;
    if ($entries["count"] <> 1) {
        echo "I'm confused. There is not one sambaDomain and one only in ldap<BR>" ;
        return "" ;
    }
    $sambaSID=$entries[0]["sambasid"][0] . sprintf("-%d", $userid * 2 + 1000)  ; 
    return $sambaSID ;
}

?>
